function [OutCellArray, Outconfig]=Map2Zernike(InCellArray, Inconfig)
%Map2ZernikeRMS computes a set of zernike coefficients from a map.  The
%coefficients are returned in ZEMAX order, and then the polynomials are
%recursively generated for fitting.  Both annular and standard zernikes may
%be fit.
%
%Inputs: Map
%Outputs: Zernike
%
%Author: Benjamin Lewis
%Email: blewis@email.arizona.edu
%
% Version history:
% 10/30/11 - Version 2
% 11/14/11 - Version 2.1

%Sources:
%http://www.opticsinfobase.org/view_article.cfm?gotourl=http%3A%2F%2Fwww%2E%opticsinfobase%2Eorg%2FDirectPDFAccess%2FEAB44FAD%2DB619%2D1513%2D9A5B781FFB97C450%5F57923%2Epdf%3Fda%3D1%26id%3D57923%26seq%3D0%26mobile%3Dno&org=University%20of%20Arizona


Outconfig=Inconfig;



OutCellArray{1}.header={'Date Created', ''};



xsize=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2};
ysize=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2};

if str2num(xsize)~=str2num(ysize)
    sizedefault=Inconfig{strcmpi(Inconfig, 'Size Mismatch'),2};
    
    if strcmpi(sizedefault,'X')
        sizedefault=['X (',xsize,')'];
    elseif strcmpi(sizedefault,'Y')
        sizedefault=['Y (',xsize,')'];
    end
    
    sizedefault=questdlg('X & Y Size Mismatch, which size should be used as the radius of the data?:','Map2ZernikeRMS',...
        ['X (',xsize,')'],['Y (',ysize,')'],'Other',sizedefault);
    
    if strcmpi(sizedefault,['X (',xsize,')'])
        sizedefault='X';
    elseif strcmpi(sizedefault,['Y (',xsize,')'])
        sizedefault='Y';
    end
    
    Outconfig{strcmpi(Outconfig, 'Size Mismatch'),2}=sizedefault;
    
    
    switch sizedefault
        case ['X (',xsize,')']
            radius=str2num(xsize);
        case ['Y (',ysize,')']
            radius=str2num(ysize);
        case 'Other'
            temps=Inconfig{strcmpi(Inconfig, 'Size'),2};
            
            validinputs=0;
            
            %Get user input
            while ~validinputs
                answers = inputdlg({'Radius:'},'Map2ZernikeRMS',1,{temps});
                if ~isempty(answers)
                    temps=answers{:};
                    temps=str2num(temps);
                    if ~isempty(temps)
                        validinputs=1;
                        radius=temps;
                    else
                        msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
                    end
                else
                    OutCellArray={};
                    return; %If answers is empty, user canceled
                end
            end
            
            Outconfig{strcmpi(Outconfig, 'Size'),2}=num2str(radius);
    end
else
    radius=str2num(xsize);
end

xunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Unit'),2};
yunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Unit'),2};
zunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Z Unit'),2};

if ~strcmpi(xunit,yunit)
    unitdefault=Inconfig{strcmpi(Inconfig, 'Unit Mismatch'),2};
    
    if strcmpi(unitdefault,'X')
        unitdefault=['X (',xunit,')'];
    elseif strcmpi(unitdefault,'Y')
        unitdefault=['Y (',yunit,')'];
    end
    
    unitdefault=questdlg('X & Y Unit Mismatch, which unit should be used?:','Map2ZernikeRMS',...
        ['X (',xunit,')'],['Y (',yunit,')'],'Other',unitdefault);
    
    if strcmpi(unitdefault,['X (',xunit,')'])
        unitdefault='X';
    elseif strcmpi(unitdefault,['Y (',yunit,')'])
        unitdefault='Y';
    end
    
    Outconfig{strcmpi(Outconfig, 'Unit Mismatch'),2}=unitdefault;
    
    
    switch unitdefault
        case ['X (',xunit,')']
            r_unit=xunit;
        case ['Y (',yunit,')']
            r_unit=yunit;
        case 'Other'
            unit=Inconfig{strcmpi(Inconfig, 'Unit'),2};
            
            answers = inputdlg({'Unit:'},'Map2ZernikeRMS',1,{unit});
            if ~isempty(answers)
                r_unit=answers{:};
            else
                OutCellArray={};
                return; %If answers is empty, user canceled
            end
            
            Outconfig{strcmpi(Outconfig, 'Unit'),2}=r_unit;
    end
else
    r_unit=xunit;
end

h_unit=zunit;

OutCellArray{1}.header(end+1,:)={'Radius', num2str(radius)};
OutCellArray{1}.header(end+1,:)={'Radius unit', r_unit};
OutCellArray{1}.header(end+1,:)={'Height unit', h_unit};




dmap=InCellArray{1}.data;



terms=Inconfig{strcmpi(Inconfig, 'Terms'),2};

validinputs=0;

%Get user input
while ~validinputs
    answers = inputdlg({'Number of Terms to Fit:'},'Map2ZernikeRMS',1,{terms});
    if ~isempty(answers)
        tempterms=answers{:};
        tempterms=str2num(tempterms);
        if ~isempty(tempterms)&&(tempterms==floor(tempterms))
            validinputs=1;
            terms=tempterms;
        else
            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
        end
    else
        OutCellArray={};
        return; %If answers is empty, user canceled
    end
end

Outconfig{strcmpi(Outconfig, 'Terms'),2}=num2str(terms);

%OutCellArray{1}.header(end+1,:)={'Terms', num2str(terms)};




% gsterms=Inconfig{strcmpi(Inconfig, 'GS Terms'),2};
%
% validinputs=0;
%
% %Get user input
% while ~validinputs
%     answers = inputdlg({'Number of Gram-Schmidt Terms to Fit:'},'Map2ZernikeRMS',1,{gsterms});
%     if ~isempty(answers)
%         tempgsterms=answers{:};
%         tempgsterms=str2num(tempgsterms);
%         if ~isempty(tempgsterms)&&(tempgsterms==floor(tempgsterms))
%             validinputs=1;
%             gsterms=tempgsterms;
%
%             if gsterms>terms
%                 gsterms=terms;
%             end
%         else
%             msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
%         end
%     else
%         OutCellArray={};
%         return; %If answers is empty, user canceled
%     end
% end
%
% Outconfig{strcmpi(Outconfig, 'GS Terms'),2}=num2str(gsterms);






ztypedefault=Inconfig{strcmpi(Inconfig, 'Z Type'),2};

ztype=questdlg('Select Zernike Type:','Map2ZernikeRMS','Annular','Standard',ztypedefault);

Outconfig{strcmpi(Outconfig, 'Z Type'),2}=ztype;


switch ztype
    case 'Standard'
        epsilon=0;
        OutCellArray{1}.header(end+1,:)={'Obscuration Ratio', num2str(epsilon)};
    case 'Annular'
        obsdefault=Inconfig{strcmpi(Inconfig, 'Obscuration'),2};
        
        obsdefault=questdlg('Select Obscuration Ratio Method:','Map2ZernikeRMS','Auto','Manual',obsdefault);
        
        Outconfig{strcmpi(Outconfig, 'Obscuration'),2}=obsdefault;
        
        
        switch obsdefault
            case 'Auto'
                epsilon=ObscurationFinder(dmap);
                OutCellArray{1}.header(end+1,:)={'Obscuration Ratio', num2str(epsilon)};
            case 'Manual'
                epsilon=Inconfig{strcmpi(Inconfig, 'Epsilon'),2};
                
                validinputs=0;
                
                %Get user input
                while ~validinputs
                    answers = inputdlg({'Obscuration Ratio:'},'Map2ZernikeRMS',1,{epsilon});
                    if ~isempty(answers)
                        tempobs=answers{:};
                        tempobs=str2num(tempobs);
                        if ~isempty(tempobs)
                            validinputs=1;
                            epsilon=tempobs;
                        else
                            msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
                        end
                    else
                        OutCellArray={};
                        return; %If answers is empty, user canceled
                    end
                end
                
                Outconfig{strcmpi(Outconfig, 'Epsilon'),2}=num2str(epsilon);
                
                OutCellArray{1}.header(end+1,:)={'Obscuration Ratio', num2str(epsilon)};
        end
end


OutCellArray{1}.type='zernike';





x=linspace(-1, 1, size(dmap,2));
y=linspace(-1, 1, size(dmap,1));

[x,y]=meshgrid(x,-1*y');

[theta, rho] = cart2pol(x,y);

for ii=1:length(theta(:))
    if theta(ii)>pi
        theta(ii)=theta(ii)-2*pi;
    end
end


%weightmap=ones(size(dmap,1),size(dmap,2));
%weightmap=WeightMapper(dmap);



% pixelspacing=(x(1,2)-x(1,1)-y(2,1)+y(1,1))/4;
%
% % bwmap=ones(size(dmap,1),size(dmap,2));
% % bwmap(isnan(dmap))=0;
% % %bwmap=conv2(bwmap,[1,1,1;1,0,1;1,1,1],'same');
% %
% % edgepixels=sum(bwmap(:,1))+sum(bwmap(:,end))+sum(bwmap(1,:))+sum(bwmap(end,:));
% %
% % %if sum(bwmap(bwmap==3))/3>=2
% % if edgepixels<=8
% %     rho=rho*(1+pixelspacing);
% % end
%
% weightmap=WeightMapper2(dmap,rho,theta,pixelspacing,epsilon);
%
% %dmap2=dmap;
%
% for ii=1:size(weightmap,1)
%     for jj=1:size(weightmap,2)
%         if isnan(dmap(ii,jj))&&weightmap(ii,jj)~=0
%              weightmap=moveweight(ii,jj,weightmap,dmap);
%              %dmap2=nnavg(ii,jj,dmap,dmap2);
%         end
%     end
% end
%
% %dmap=dmap2;
%
% %weightmap=ones(size(dmap,1),size(dmap,2));



wdefault=Inconfig{strcmpi(Inconfig, 'Weight Map'),2};

wm=questdlg('Use Weight Map?','Map2ZernikeRMS','Yes','No',wdefault);

Outconfig{strcmpi(Outconfig, 'Weight Map'),2}=wm;


switch wm
    case 'Yes'
        pixelspacing=(x(1,2)-x(1,1)-y(2,1)+y(1,1))/4;
        
        [weightmap,rho,theta]=WeightMapper2(dmap,rho,theta,pixelspacing,epsilon);
        
        for ii=1:size(weightmap,1)
            for jj=1:size(weightmap,2)
                if isnan(dmap(ii,jj))&&weightmap(ii,jj)~=0
                    [weightmap,rho,theta]=moveweight(ii,jj,weightmap,dmap,rho,theta);
                    %weightmap=moveweight(ii,jj,weightmap,dmap);
                end
            end
        end
        
        [x,y]=pol2cart(theta,rho);
        
        weightmap(isnan(dmap))=[];
        
    case 'No'
        weightmap=[];
        
        gsterms=Inconfig{strcmpi(Inconfig, 'GS Terms'),2};
        
        validinputs=0;
        
        %Get user input
        while ~validinputs
            answers = inputdlg({'Number of Gram-Schmidt Terms to Fit:'},'Map2ZernikeRMS',1,{gsterms});
            if ~isempty(answers)
                tempgsterms=answers{:};
                tempgsterms=str2num(tempgsterms);
                if ~isempty(tempgsterms)&&(tempgsterms==floor(tempgsterms))
                    validinputs=1;
                    gsterms=tempgsterms;
                    
                    if gsterms>terms
                        gsterms=terms;
                    end
                else
                    msgbox('Please enter valid inputs','Invalid Inputs','warn','modal')
                end
            else
                OutCellArray={};
                return; %If answers is empty, user canceled
            end
        end
        
        Outconfig{strcmpi(Outconfig, 'GS Terms'),2}=num2str(gsterms);
end


theta(isnan(dmap))=[];
rho(isnan(dmap))=[];

dmap(isnan(dmap))=[];

if isempty(weightmap)
    [zvec0,mvec,nvec]=ComputeCoeffsGS(rho, theta, terms, gsterms, epsilon, dmap);
else
    [zvec0,mvec,nvec]=ComputeCoeffs(rho, theta, terms, epsilon, dmap, weightmap);
end

OutCellArray{1}.data=[zvec0(:),mvec(:),nvec(:)];

%     function wm=WeightMapper(dmap)
%         wm=ones(size(dmap,1),size(dmap,2));
%
%         wm(isnan(dmap))=0;
%
%         wm=conv2(wm,[1,1,1;1,0,1;1,1,1],'same');
%
%         if sum(wm(wm==3))<=2
%             wm(wm<=2)=0;
%             wm(wm<6&wm>2)=.5;
%             wm(wm>=6)=1;
%         else
%             wm(wm<=2)=0;
%             wm(wm==3)=.5;
%             wm(wm==4)=.5;
%             wm(wm==5)=1;
%             wm(wm==6)=1;
%             wm(wm>=6)=1;
%         end
%
%         wm(isnan(dmap))=0;
%     end

    function [wm,rho,theta]=moveweight(ii,jj,wm,dmap,rho,theta)
    %function wm=moveweight(ii,jj,wm,dmap)
        
%         if ii==89&&jj==67
%             pause;
%         end
    
        if jj-1>0&&wm(ii,jj-1)~=0&&~isnan(dmap(ii,jj-1)), ...
                ln=1; else ln=0; end
        
        if jj+1<=size(wm,2)&&wm(ii,jj+1)~=0&&~isnan(dmap(ii,jj+1)), ...
                rn=1; else rn=0; end
        
        if ii-1>0&&wm(ii-1,jj)~=0&&~isnan(dmap(ii-1,jj)), ...
                un=1; else un=0; end
        
        if ii+1<=size(wm,1)&&wm(ii+1,jj)~=0&&~isnan(dmap(ii+1,jj)), ...
                dn=1; else dn=0; end
        
        nbrs=ln+rn+un+dn;
        
        if nbrs~=0
            w=wm(ii,jj)/nbrs;
            
            if ln
                wm(ii,jj-1)=wm(ii,jj-1)+w;
                [a,b]=pol2cart(theta(ii,jj-1),rho(ii,jj-1));
                [c,d]=pol2cart(theta(ii,jj),rho(ii,jj));
                [tt,tr]=cart2pol((a+w*c)/(1+w),(b+w*d)/(1+w));
                rho(ii,jj-1)=tr;
                theta(ii,jj-1)=tt;
            end
            
            if rn
                wm(ii,jj+1)=wm(ii,jj+1)+w;
                [a,b]=pol2cart(theta(ii,jj+1),rho(ii,jj+1));
                [c,d]=pol2cart(theta(ii,jj),rho(ii,jj));
                [tt,tr]=cart2pol((a+w*c)/(1+w),(b+w*d)/(1+w));
                rho(ii,jj+1)=tr;
                theta(ii,jj+1)=tt;
            end
            
            if un
                wm(ii-1,jj)=wm(ii-1,jj)+w;
                [a,b]=pol2cart(theta(ii-1,jj),rho(ii-1,jj));
                [c,d]=pol2cart(theta(ii,jj),rho(ii,jj));
                [tt,tr]=cart2pol((a+w*c)/(1+w),(b+w*d)/(1+w));
                rho(ii-1,jj)=tr;
                theta(ii-1,jj)=tt;
            end
            
            if dn
                wm(ii+1,jj)=wm(ii+1,jj)+w;
                [a,b]=pol2cart(theta(ii+1,jj),rho(ii+1,jj));
                [c,d]=pol2cart(theta(ii,jj),rho(ii,jj));
                [tt,tr]=cart2pol((a+w*c)/(1+w),(b+w*d)/(1+w));
                rho(ii+1,jj)=tr;
                theta(ii+1,jj)=tt;
            end
        end
        
        wm(ii,jj)=NaN;
    end

%     function dmap2=nnavg(ii,jj,dmap,dmap2)
%         if jj-1>0&&~isnan(dmap(ii,jj-1)), ...
%                 ln=1; else ln=0; end
%
%         if jj+1<=size(dmap,2)&&~isnan(dmap(ii,jj+1)), ...
%                 rn=1; else rn=0; end
%
%         if ii-1>0&&~isnan(dmap(ii-1,jj)), ...
%                 un=1; else un=0; end
%
%         if ii+1<=size(dmap,1)&&~isnan(dmap(ii+1,jj)), ...
%                 dn=1; else dn=0; end
%
%         nbrs=ln+rn+un+dn;
%
%         if nbrs~=0
%             v=0;
%
%             if ln, v=v+dmap(ii,jj-1); end
%
%             if rn, v=v+dmap(ii,jj+1); end
%
%             if un, v=v+dmap(ii-1,jj); end
%
%             if dn, v=v+dmap(ii+1,jj); end
%
%             dmap2(ii,jj)=v/nbrs;
%         end
%     end

    function [wm,rho,theta]=WeightMapper2(dmap,rho,theta,pixelspacing,epsilon)
        ps=(sqrt(2)/2)*pixelspacing;
        
        wm=zeros(size(dmap));
        
        for aa=1:length(wm(:))
            if rho(aa)>epsilon-ps&&rho(aa)<epsilon+ps
                [xi,yi]=pol2cart(theta(aa),rho(aa));
                
                x=linspace(-pixelspacing/2,pixelspacing/2,10);
                [x,y]=meshgrid(x,-1*x');
                x=x+xi;
                y=y+yi;
                
                [thetatest,rhotest]=cart2pol(x,y);
                
                rhobool=rhotest;
                
                rhobool(rhotest<=epsilon)=0;
                rhobool(rhotest>epsilon)=1;
                
                
                
                if theta(aa)+pi>.0001 %Removes an ambiguity from near the negative of the polar axis
                    theta(aa)=mean(thetatest(rhobool==1));
                end
                
                rho(aa)=mean(rhotest(rhobool==1));
                
                
                
                wm(aa)=sum(rhobool(:))/length(rhobool(:));
            elseif rho(aa)>epsilon+ps&&rho(aa)<1-ps
                wm(aa)=1;
                
            elseif rho(aa)>1-ps&&rho(aa)<1+ps
                [xi,yi]=pol2cart(theta(aa),rho(aa));
                
                x=linspace(-pixelspacing/2,pixelspacing/2,10);
                [x,y]=meshgrid(x,-1*x');
                x=x+xi;
                y=y+yi;
                
                [thetatest,rhotest]=cart2pol(x,y);
                
                rhobool=rhotest;
                
                rhobool(rhotest<=1)=1;
                rhobool(rhotest>1)=0;
                
                
                
                if theta(aa)+pi>.0001 %Removes an ambiguity from near the negative of the polar axis
                    theta(aa)=mean(thetatest(rhobool==1));
                end
                
                rho(aa)=mean(rhotest(rhobool==1));
                
                
                
                wm(aa)=sum(rhobool(:))/length(rhobool(:));
            end
        end
    end


    function [epsilon] = ObscurationFinder(dmap)
        %UNTITLED Summary of this function goes here
        %   Detailed explanation goes here
        
        bwmap=dmap;
        
        bwmap(isnan(dmap))=0;
        bwmap(~isnan(dmap))=1;
        
        closemap=imfill(bwmap);
        
        holemap=xor(closemap,bwmap);
        
        x=linspace(-1, 1, size(dmap,2));
        y=linspace(-1, 1, size(dmap,1));
        
        [x,y]=meshgrid(x,-1*y');
        
        [~, rho] = cart2pol(x,y);
        
        rho(holemap)=NaN;
        
        rho=rho(:);
        rho(isnan(rho))=[];
        
        rho=unique(rho);
        
        %epsilon=rho(1)*(1-1/size(dmap,1)); %correction for overestimation by a pixel
        epsilon=rho(1);
    end


    function [zout,mout,nout]=ComputeCoeffs(rho, theta, maxTerm, epsilon, dmap, weightmap)
        m=0;
        n=0;
        
        tt=1;
        indices(1,:)=[0,0,0];
        
        while tt<maxTerm
            if m==n||m==-1*n
                n=n+1;
                if mod(n,2)==0, m=0; else m=1; end
            else
                m=m+2;
            end
            if m==0
                indices(tt+1,:)=[m, n, (n-abs(m))/2]; %We can't index to 0, so zernike z0 maps to 1
            else
                if mod(tt,2)~=0 %Strangely enough, ZEMAX uses this to decide the order of terms
                    indices(tt+1,:)=[m, n, (n-abs(m))/2];
                    tt=tt+1;
                    indices(tt+1,:)=[-1*m, n, (n-abs(m))/2];
                else
                    indices(tt+1,:)=[-1*m, n, (n-abs(m))/2];
                    tt=tt+1;
                    indices(tt+1,:)=[m, n, (n-abs(m))/2];
                end
            end
            
            tt=tt+1;
        end
        
        indices=indices(1:maxTerm,:);
        
        if size(indices,1)>1
            Maxs=max(abs(indices));
            MaxM=Maxs(1);
            MaxJ=Maxs(3);
        else
            MaxM=abs(indices(1));
            MaxJ=abs(indices(3));
        end
        
        ThisRow=cell(1,MaxM+MaxJ-1); %This is going the Q Polynomials
        LastRow=cell(1,MaxM+MaxJ-1); %This is going the Q Polynomials
        
        thisH=zeros(1,MaxM+MaxJ-1);
        lastH=zeros(1,MaxM+MaxJ-1);
        
        zout=zeros(maxTerm, 1);
        mout=zeros(maxTerm, 1);
        nout=zeros(maxTerm, 1);
        
        
        %         rho=rho(:);
        %         theta=theta(:);
        %         dmap=dmap(:);
        tdmap=dmap;
        
        rho(end+1)=0;
        theta(end+1)=0;
        
        
        X=(2.*((rho.^2-epsilon^2)./(1-epsilon^2)))-1;
        X(isnan(rho))=NaN;
        
        for mm=0:MaxM
            Qsum=zeros(size(rho));
            for jj=0:MaxM+MaxJ-1-(mm-1)
                if mm==0
                    if jj==0
                        TempQ=ones(size(rho));
                        TempQ(isnan(rho))=NaN;
                        ThisRow{jj+1}=TempQ;
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    elseif jj==1
                        ThisRow{jj+1}=X;
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    else
                        ThisRow{jj+1}=(2*(jj-1)+1).*X.*ThisRow{jj}-(jj-1).*ThisRow{jj-1};
                        ThisRow{jj+1}=ThisRow{jj+1}./((jj-1)+1);
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    end
                else
                    a=2*(2*jj+2*mm-1)*lastH(jj+1);
                    b=(jj+mm)*(1-epsilon^2)*LastRow{jj+1}(end);
                    c=a/b;
                    
                    Qsum=Qsum+LastRow{jj+1}.*LastRow{jj+1}(end)./lastH(jj+1);
                    
                    ThisRow{jj+1}=c.*Qsum;
                    
                    a=2*(2*jj+2*mm-1)*LastRow{jj+2}(end);
                    b=(jj+mm)*(1-epsilon^2)*LastRow{jj+1}(end);
                    
                    thisH(jj+1)=-1*(a/b)*lastH(jj+1);
                end
            end
            
            LastRow=ThisRow;
            lastH=thisH;
            
            rhoM=rho.^mm;
            
            zt=[];
            lvec=[];
            
            for ll=1:size(indices,1)
                if abs(indices(ll,1))==mm
                    m=indices(ll,1);
                    n=indices(ll,2);
                    j=indices(ll,3);
                    
                    scale=(1-epsilon^2)/(2*(2*j+mm+1)*thisH(j+1));
                    scale=scale^.5;
                    scale=scale.*rhoM;
                    
                    if m==0 %Multiply by cos or sin if nessicary, and normalize so RMS=1
                        zernike=sqrt(n+1).*scale.*ThisRow{j+1};
                    elseif m>0
                        zernike=sqrt(2)*sqrt(n+1).*scale.*ThisRow{j+1}.*cos(mm.*theta);
                    else
                        zernike=sqrt(2)*sqrt(n+1).*scale.*ThisRow{j+1}.*sin(mm.*theta);
                    end
                    
                    mout(ll)=m;
                    nout(ll)=n;
                    
                    zernike(end)=[];
                    
                    if isempty(zt)
                        zt{1}=zernike;
                        lvec(1)=ll;
                    else
                        zt{end+1}=zernike;
                        lvec(end+1)=ll;
                    end
                    
                end
            end
            
            [terms,tdmap]=GS(zt,tdmap,weightmap);
            
            for qq=1:length(terms)
                zout(lvec(qq))=terms(qq);
            end
        end
        
        
        function zt=proj(z1,z2,weightmap)
            zt=(nansum(nansum(z1.*z2.*weightmap))/nansum(nansum(z2.*z2.*weightmap)));
        end
        
        %Gram-Schmidt
        function [terms,tdmap]=GS(zc,tdmap,weightmap)
            cmat=zeros(length(zc));
            
            for kk=1:length(zc)
                cmat(kk,kk)=1;
                
                for rr=1:kk-1
                    B=proj(zc{kk},zc{rr},weightmap);
                    
                    zc{kk}=zc{kk}-B*zc{rr};
                    
                    cmat(rr,kk)=-1*B;
                end
                
                tmap=zc{kk}.*tdmap.*weightmap;
                
                a=sum(sum(tmap));
                
                z2=zc{kk}.^2.*weightmap;
                
                b=sum(sum(z2));
                
                z=a/b;
                
                tdmap=tdmap-z.*zc{kk};
                
                terms(kk)=z;
            end
            
            terms=cmat*terms';
        end
        
    end





    function [zout,mout,nout]=ComputeCoeffsGS(rho, theta, maxTerm, gsterms, epsilon, dmap)
        m=0;
        n=0;
        
        tt=1;
        indices(1,:)=[0,0,0];
        
        while tt<maxTerm
            if m==n||m==-1*n
                n=n+1;
                if mod(n,2)==0, m=0; else m=1; end
            else
                m=m+2;
            end
            if m==0
                indices(tt+1,:)=[m, n, (n-abs(m))/2]; %We can't index to 0, so zernike z0 maps to 1
            else
                if mod(tt,2)~=0 %Strangely enough, ZEMAX uses this to decide the order of terms
                    indices(tt+1,:)=[m, n, (n-abs(m))/2];
                    tt=tt+1;
                    indices(tt+1,:)=[-1*m, n, (n-abs(m))/2];
                else
                    indices(tt+1,:)=[-1*m, n, (n-abs(m))/2];
                    tt=tt+1;
                    indices(tt+1,:)=[m, n, (n-abs(m))/2];
                end
            end
            
            tt=tt+1;
        end
        
        indices=indices(1:maxTerm,:);
        
        if size(indices,1)>1
            Maxs=max(abs(indices));
            MaxM=Maxs(1);
            MaxJ=Maxs(3);
        else
            MaxM=abs(indices(1));
            MaxJ=abs(indices(3));
        end
        
        ThisRow=cell(1,MaxM+MaxJ-1); %This is going the Q Polynomials
        LastRow=cell(1,MaxM+MaxJ-1); %This is going the Q Polynomials
        
        thisH=zeros(1,MaxM+MaxJ-1);
        lastH=zeros(1,MaxM+MaxJ-1);
        
        zout=zeros(maxTerm, 1);
        mout=zeros(maxTerm, 1);
        nout=zeros(maxTerm, 1);
        
        
        zc=cell(1,gsterms);
        
        %         rho=rho(:);
        %         theta=theta(:);
        %         dmap=dmap(:);
        tdmap=dmap;
        
        rho(end+1)=0;
        theta(end+1)=0;
        
        
        X=(2.*((rho.^2-epsilon^2)./(1-epsilon^2)))-1;
        X(isnan(rho))=NaN;
        
        for mm=0:MaxM
            Qsum=zeros(size(rho));
            for jj=0:MaxM+MaxJ-1-(mm-1)
                if mm==0
                    if jj==0
                        TempQ=ones(size(rho));
                        TempQ(isnan(rho))=NaN;
                        ThisRow{jj+1}=TempQ;
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    elseif jj==1
                        ThisRow{jj+1}=X;
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    else
                        ThisRow{jj+1}=(2*(jj-1)+1).*X.*ThisRow{jj}-(jj-1).*ThisRow{jj-1};
                        ThisRow{jj+1}=ThisRow{jj+1}./((jj-1)+1);
                        thisH(jj+1)=(1-epsilon^2)/(2*(2*jj+1));
                    end
                else
                    a=2*(2*jj+2*mm-1)*lastH(jj+1);
                    b=(jj+mm)*(1-epsilon^2)*LastRow{jj+1}(end);
                    c=a/b;
                    
                    Qsum=Qsum+LastRow{jj+1}.*LastRow{jj+1}(end)./lastH(jj+1);
                    
                    ThisRow{jj+1}=c.*Qsum;
                    
                    a=2*(2*jj+2*mm-1)*LastRow{jj+2}(end);
                    b=(jj+mm)*(1-epsilon^2)*LastRow{jj+1}(end);
                    
                    thisH(jj+1)=-1*(a/b)*lastH(jj+1);
                end
            end
            
            LastRow=ThisRow;
            lastH=thisH;
            
            rhoM=rho.^mm;
            
            for ll=1:size(indices,1)
                if abs(indices(ll,1))==mm
                    m=indices(ll,1);
                    n=indices(ll,2);
                    j=indices(ll,3);
                    
                    scale=(1-epsilon^2)/(2*(2*j+mm+1)*thisH(j+1));
                    scale=scale^.5;
                    scale=scale.*rhoM;
                    
                    if m==0 %Multiply by cos or sin if nessicary, and normalize so RMS=1
                        zernike=sqrt(n+1).*scale.*ThisRow{j+1};
                    elseif m>0
                        zernike=sqrt(2)*sqrt(n+1).*scale.*ThisRow{j+1}.*cos(mm.*theta);
                    else
                        zernike=sqrt(2)*sqrt(n+1).*scale.*ThisRow{j+1}.*sin(mm.*theta);
                    end
                    
                    mout(ll)=m;
                    nout(ll)=n;
                    
                    zernike(end)=[];
                    
                    
                    
                    tmap=zernike.*tdmap;
                    %tmap(isnan(tmap))=0;
                    
                    %a=trapz(trapz(tmap));
                    a=sum(sum(tmap));
                    
                    
                    
                    z2=zernike.^2;
                    %z2(isnan(z2))=0;
                    
                    %b=trapz(trapz(z2));
                    b=sum(sum(z2));
                    
                    z=a/b;
                    
                    
                    zout(ll)=z;
                    
                    tdmap=tdmap-z.*zernike;
                    
                    
                    
                    
                    
                    if ll<=gsterms
                        zc{ll}=zernike;
                    end
                end
            end
        end
        
        
        tdmap=dmap;
        
        
        function zt=proj(z1,z2)
            zt=(nansum(nansum(z1.*z2))/nansum(nansum(z2.*z2)));
        end
        
        %Gram-Schmidt
        cmat=zeros(gsterms);
        
        for kk=1:gsterms
            cmat(kk,kk)=1;
            
            for jj=1:kk-1
                B=proj(zc{kk},zc{jj});
                
                zc{kk}=zc{kk}-B*zc{jj};
                
                cmat(jj,kk)=-1*B;
            end
            
            tmap=zc{kk}.*tdmap;
            
            %tmap=zc{kk}.*tdmap;
            
            
            %tmap(isnan(tmap))=0;
            
            %a=trapz(trapz(tmap));
            a=sum(sum(tmap));
            
            
            
            z2=zc{kk}.^2;
            
            %z2=zc{kk}.^2;
            
            
            %z2(isnan(z2))=0;
            
            %b=trapz(trapz(z2));
            b=sum(sum(z2));
            
            z=a/b;
            
            tdmap=tdmap-z.*zc{kk};
            
            tzout(kk)=z;
        end
        
        if gsterms~=0
            tzout2=cmat*tzout';
            
            zout(1:gsterms)=tzout2;
        end
    end
end










