function [OutCellArray, Outconfig]=PSD(InCellArray, Inconfig)
%PSD calculates the power spectrum density of a map
%
%Inputs: Map
%Outputs: Frequency Map
%
%Author: Ping Zhou
%Email:
%
% Version history:
% 9/9/10 - Version 1

OutCellArray{1}=InCellArray{1};
OutCellArray{1}.type='freqmap';

xsize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Size'),2});
ysize=str2double(InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Size'),2});

xunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'X Unit'),2};
yunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Y Unit'),2};
zunit=InCellArray{1}.header{strcmpi(InCellArray{1}.header, 'Z Unit'),2};

OutCellArray{1}.header = [];
OutCellArray{1}.header={'Date Created', ''};

OutCellArray{1}.header(end+1,:)={'X unit', ['1/', xunit]};
OutCellArray{1}.header(end+1,:)={'Y unit', ['1/', yunit]};
OutCellArray{1}.header(end+1,:)={'Z unit', zunit};

delx = xsize/size(InCellArray{1}.data, 2);          % size of each pixel in x-direction
xsize = 1/(2*delx);                                 % max frequency in x direction

OutCellArray{1}.header(end+1,:)={'X Max Frequency', num2str(xsize)};

dely = ysize/size(InCellArray{1}.data, 1);
ysize = 1/(2*dely);

OutCellArray{1}.header(end+1,:)={'Y Max Frequency', num2str(ysize)};

Outconfig=Inconfig;

data = InCellArray{1}.data;

data(isnan(data))=0;

G = delx*dely*fftshift(fft2(fftshift(data)));
G = G.*conj(G);

OutCellArray{1}.data=G;

